==> clipgrab-3.2.0.9.tar.bz2: clipgrab-3.2.0.9/video_youtube.cpp <==
/*
    ClipGrab³
    Copyright (C) Philipp Schmieder
    http://clipgrab.de
    feedback [at] clipgrab [dot] de

    This file is part of ClipGrab.
    ClipGrab is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    ClipGrab is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ClipGrab.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "video_youtube.h"

video_youtube::video_youtube()
{
    this->_name = "Youtube";
    this->_supportsTitle = true;
    this->_supportsDescription = true;
    this->_supportsThumbnail = true;
    this->_supportsSearch = true;
    this->_icon = 0;
    this->_urlRegExp << QRegExp("http[s]?://\\w*\\.youtube\\.com/watch.*v\\=.*", Qt::CaseInsensitive);
    this->_urlRegExp << QRegExp("http[s]?://\\w*\\.youtube\\.com/view_play_list\\?p\\=.*&v\\=.*", Qt::CaseInsensitive);
    this->_urlRegExp << QRegExp("http[s]?://youtu.be/.*", Qt::CaseInsensitive);
}

video* video_youtube::createNewInstance()
{
    return new video_youtube();
}

bool video_youtube::setUrl(QString url)
{
    if (_url.isEmpty())
    {
        QString x = "#!";
        QString y = "?";
        url.replace(x, y);
        this->_url = QUrl(url);
        if (_url.isValid())
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

void video_youtube::parseVideo(QString html)
{

    QRegExp expression;

    expression = QRegExp("<meta name=\"title\" content=\"(.*)\"");
    expression.setMinimal(true);
    if (expression.indexIn(html) !=-1)
    {
        _title = QString(expression.cap(1)).replace("&amp;quot;", "\"").replace("&amp;amp;", "&").replace("&#39;", "'").replace("&quot;", "\"");
        expression = QRegExp("\"url_encoded_fmt_stream_map\": \"(.*)\"");
        expression.setMinimal(true);
        if (expression.indexIn(html)!=-1 && expression.cap(1) != "")
        {
            QStringList qualityLinks =  QUrl::fromEncoded(expression.cap(1).toAscii()).toString(QUrl::None).split(",itag=");
            //QStringList qualityLinks = QUrl::fromEncoded(expression.cap(1).toAscii()).toString(QUrl::None).replace("%2C",",").replace("\\u0026", "&").replace("&sig=", "&signature=").split("url=");
            qDebug() << qualityLinks;
            QStringList supportedFmts;
            QStringList fmtQualities;


            
            QSettings settings;
            if (settings.value("UseWebM", false).toBool() == false)
            {
                supportedFmts <<    "37"            <<  "22"            << "35"             << "34"             <<  "5";
                fmtQualities <<     tr("HD (1080p)")    <<  tr("HD (720p)")     << tr("high (480p)")    << tr("normal (360p)")  <<  tr("low (240p)");
            }
            else
            {
                supportedFmts <<	"46" << "45"	<< "44" << "43";
                fmtQualities <<		tr("HD (1080p)") << tr("HD (720p)") << tr("high (480p)") << "normal (360p)";
            }
			
            for (int i=0; i < qualityLinks.size(); i++)
            {


                QRegExp formatExpression;
                formatExpression = QRegExp("mp4|webm|flv");
                formatExpression.setMinimal(true);

                QRegExp urlExpression;
                urlExpression = QRegExp("(http[s]?.*)\\\\u0026");
                urlExpression.setMinimal(true);

                QRegExp sigExpression;
                sigExpression = QRegExp("sig=(.*)\\\\u0026");
                sigExpression.setMinimal(true);

                for (int ii=0; ii < supportedFmts.size(); ii++)
                {

                    if (qualityLinks.at(i).contains("itag="+supportedFmts.at(ii)) && formatExpression.indexIn(qualityLinks.at(i)) > -1 && urlExpression.indexIn(qualityLinks.at(i)) > -1 && sigExpression.indexIn(qualityLinks.at(i)) > -1 )
                    {
                        videoQuality newQuality;
                        newQuality.quality = fmtQualities.at(ii);
                        newQuality.containerName = "." + formatExpression.cap(0);
                        newQuality.videoUrl = QUrl::fromEncoded(urlExpression.cap(1).toAscii()).toString().append("&signature=").append(sigExpression.cap(1));
                        _supportedQualities.append(newQuality);
                    }

                }
            }
        }
        else if (expression.indexIn(html)!=-1 && expression.cap(1) == "")
        {
            expression = QRegExp("\"t\": \"(.*)\"");
            expression.setMinimal(true);
            QRegExp expression2 = QRegExp("\"video_id\": \"(.*)\"");
            expression2.setMinimal(true);
            if (expression.indexIn(html) !=-1 && expression2.indexIn(html) !=-1)
            {
                videoQuality newQuality;
                newQuality.quality = tr("normal");
                newQuality.videoUrl = QUrl::fromEncoded(QString("http://www.youtube.com/get_video?video_id=" + expression2.cap(1) + "&t=" + expression.cap(1)).toAscii()).toString(QUrl::None);
                _supportedQualities.append(newQuality);
            }
            else
            {
                emit error("Could not retrieve video link.", this);

            }
        }
        else
        {
            emit error("Could not retrieve video link.", this);

        }
    }
    else
    {
        emit error("Could not retrieve video title.", this);
    }

    emit analysingFinished();
}

==> clipgrab-3.2.0.10.tar.bz2: clipgrab-3.2.0.10/video_youtube.cpp <==
/*
    ClipGrab³
    Copyright (C) Philipp Schmieder
    http://clipgrab.de
    feedback [at] clipgrab [dot] de

    This file is part of ClipGrab.
    ClipGrab is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    ClipGrab is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ClipGrab.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "video_youtube.h"

video_youtube::video_youtube()
{
    this->_name = "Youtube";
    this->_supportsTitle = true;
    this->_supportsDescription = true;
    this->_supportsThumbnail = true;
    this->_supportsSearch = true;
    this->_icon = 0;
    this->_urlRegExp << QRegExp("http[s]?://\\w*\\.youtube\\.com/watch.*v\\=.*", Qt::CaseInsensitive);
    this->_urlRegExp << QRegExp("http[s]?://\\w*\\.youtube\\.com/view_play_list\\?p\\=.*&v\\=.*", Qt::CaseInsensitive);
    this->_urlRegExp << QRegExp("http[s]?://youtu.be/.*", Qt::CaseInsensitive);
}

video* video_youtube::createNewInstance()
{
    return new video_youtube();
}

bool video_youtube::setUrl(QString url)
{
    if (_url.isEmpty())
    {
        QString x = "#!";
        QString y = "?";
        url.replace(x, y);
        this->_url = QUrl(url);
        if (_url.isValid())
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }
}

void video_youtube::parseVideo(QString html)
{

    QRegExp expression;

    expression = QRegExp("<meta name=\"title\" content=\"(.*)\"");
    expression.setMinimal(true);
    if (expression.indexIn(html) !=-1)
    {
        _title = QString(expression.cap(1)).replace("&amp;quot;", "\"").replace("&amp;amp;", "&").replace("&#39;", "'").replace("&quot;", "\"");
        expression = QRegExp("\"url_encoded_fmt_stream_map\": \"(.*)\"");
        expression.setMinimal(true);
        if (expression.indexIn(html)!=-1 && expression.cap(1) != "")
        {
            qDebug() << "aaaa";
            QStringList qualityLinks =  expression.cap(1).split(",");
            qualityLinks.replaceInStrings("\\u0026", ",");
            qDebug() << qualityLinks;
            QStringList supportedFmts;
            QStringList fmtQualities;


            
            QSettings settings;
            if (settings.value("UseWebM", false).toBool() == false)
            {
                supportedFmts <<    "37"            <<  "22"            << "35"             << "34"             <<  "5";
                fmtQualities <<     tr("HD (1080p)")    <<  tr("HD (720p)")     << tr("high (480p)")    << tr("normal (360p)")  <<  tr("low (240p)");
            }
            else
            {
                supportedFmts <<	"46" << "45"	<< "44" << "43";
                fmtQualities <<		tr("HD (1080p)") << tr("HD (720p)") << tr("high (480p)") << "normal (360p)";
            }
			
            for (int i=0; i < qualityLinks.size(); i++)
            {


                QRegExp formatExpression;
                formatExpression = QRegExp("mp4|webm|flv");
                formatExpression.setMinimal(true);

                QRegExp urlExpression;
                urlExpression = QRegExp("(http[s]?[^,]+)");

                QRegExp sigExpression;
                sigExpression = QRegExp("sig=([^,]+)");


                for (int ii=0; ii < supportedFmts.size(); ii++)
                {/*

                    qDebug() << qualityLinks.at(i);
                    qDebug() << sigExpression.indexIn(qualityLinks.at(i));
                    qDebug() << urlExpression.indexIn(qualityLinks.at(i));
                    qDebug() << formatExpression.indexIn(qualityLinks.at(i));
*/
                    

                    
                    if (qualityLinks.at(i).contains("itag="+supportedFmts.at(ii)) && formatExpression.indexIn(qualityLinks.at(i)) > -1 && urlExpression.indexIn(qualityLinks.at(i)) > -1 && sigExpression.indexIn(qualityLinks.at(i)) > -1 )
                    {
                        /*qDebug() << "###";
                        qDebug() << urlExpression.cap(1) << "&signature=" << sigExpression.cap(1);
                        qDebug() << "###";*/
                        videoQuality newQuality;
                        newQuality.quality = fmtQualities.at(ii);
                        newQuality.containerName = "." + formatExpression.cap(0);
                        

                                                                                                                                  
                        newQuality.videoUrl = QUrl::fromEncoded(QUrl::fromEncoded(urlExpression.cap(1).toAscii()).toString().toAscii()).toString().append("&signature=").append(sigExpression.cap(1));
                        _supportedQualities.append(newQuality);
                    }

                }
            }
        }
        else if (expression.indexIn(html)!=-1 && expression.cap(1) == "")
        {
            expression = QRegExp("\"t\": \"(.*)\"");
            expression.setMinimal(true);
            QRegExp expression2 = QRegExp("\"video_id\": \"(.*)\"");
            expression2.setMinimal(true);
            if (expression.indexIn(html) !=-1 && expression2.indexIn(html) !=-1)
            {
                videoQuality newQuality;
                newQuality.quality = tr("normal");
                newQuality.videoUrl = QUrl::fromEncoded(QString("http://www.youtube.com/get_video?video_id=" + expression2.cap(1) + "&t=" + expression.cap(1)).toAscii()).toString(QUrl::None);
                _supportedQualities.append(newQuality);
            }
            else
            {
                emit error("Could not retrieve video link.", this);

            }
        }
        else
        {
            emit error("Could not retrieve video link.", this);

        }
    }
    else
    {
        emit error("Could not retrieve video title.", this);
    }

    emit analysingFinished();
}
